import unittest
import numpy as np
from eksternlab import Lane
from eksternlab import extend_dataset


class TestLane(unittest.TestCase):
    def test_perfect_circle(self):
        t = np.linspace(0.1, np.pi/2.0-0.1, 500)
        x = -0.5 + np.sin(t)
        y = np.cos(t)

        lane = Lane(x, y)
        h = lane.height(smooth=5)
        self.assertTrue(np.allclose(h(x), y, atol=0.1))

        angle = lane.slope(smooth=5)
        expect = t
        self.assertTrue(np.allclose(expect, angle(x), atol=0.1))

        radius = lane.radius_of_curvature(smooth=5)
        self.assertTrue(np.allclose(radius(x)[10:-10], -1.0, atol=0.1))

    def test_extend_dataset(self):
        x = np.linspace(0.0, 2.0, 100)
        y = 2*x

        x_new, y_new = extend_dataset(x, y, -1.0, 4.0, num_points=10)
        self.assertAlmostEqual(x_new[0], -1.0)
        self.assertAlmostEqual(x_new[-1], 4.0)
        self.assertAlmostEqual(y_new[0], 2*x_new[0])
        self.assertAlmostEqual(y_new[-1], 2*x_new[-1])


if __name__ == '__main__':
    unittest.main()



from scipy.interpolate import CubicSpline
import numpy as np


def height(x, y):
    """
    Returns the height of the lane as a function of horizontal distance

    Parameters:

    x: array
        List with x-coordinates of the knots

    y: array
        List with the y-coordinates of the knots

    >>> from eksternlab import height
    >>> x = [0.0, 0.2, 0.4, 0.6, 0.8]
    >>> y = [1.0, 0.7, 0.4, 0.5, 0.6]
    >>> h = height(x, y)

    To get the height at an arbitrary position x
    
    >>> x1 = 0.23
    >>> y1 = h(x1)
    """
    return CubicSpline(x, y, bc_type='natural', extrapolate=True)


def slope(cs, x):
    """
    Return the slope of the curve

    Parameters:

    cs: CubicSpline
        Instance of a cubic spline class
    x: float or array of floats
        Points at which to evaluate the slope angle

    >>> from eksternlab import slope
    >>> x = [0.23, 0.26, 0.39]
    >>> alpha = slope(h, x)
    """
    values = cs(x, 1)
    return -np.arctan(values)


def curvature(cs, x):
    """
    Return the curvature (inverse radius of curvature) of the lane.

    Parameters:

    cs: CubicSpline
        Instance of the CubicSpline calss

    x: float or array of floats
        Points at which to evaluate the curvature

    >>> from eksternlab import curvature
    >>> x = [0.23, 0.26, 0.39]
    >>> kappa = curvature(h, x)
    """
    second_deriv = cs(x, 2)
    deriv = cs(x, 1)
    return second_deriv/(1.0 + deriv**2)**1.5



import numpy as np
from scipy.signal import savgol_filter
from scipy.interpolate import interp1d


class Lane(object):
    """
    Utility class for extracting useful properties of a lane. It is assumed
    that there is a one-to-one mapping between the x- and y-coordinates.

    Parameters:

    x: array
        Array with x coordinates

    y: array
        Array with height coordinates of the lane
    """
    def __init__(self, x, y):
        self.x = np.linspace(np.min(x), np.max(x), len(x))
        interpolator = interp1d(x, y)
        self.y = interpolator(self.x)

    @property
    def dx(self):
        return self.x[1] - self.x[0]

    def height(self, smooth=5):
        """
        Returns an interpolator for the height of the lane

        Parameters:

        smooth: int
            Window length in Savgol-Filter. Has to be odd.
        """
        if smooth < 5:
            raise ValueError("smooth parameter has to be larger than 5")

        if smooth % 2 == 0:
            raise ValueError('smooth parameter has to be an odd number')

        filtered_data = savgol_filter(self.y, smooth, 3)
        return interp1d(self.x, filtered_data, bounds_error=False,
                        fill_value='extrapolate', assume_sorted=True)

    def slope(self, smooth=5):
        """
        Returns an interpolator for the slope angle (in radians) of the lane

        Parameters:

        smooth: int
            Window length in Savgol filter. Has to be odd.
        """
        if smooth < 5:
            raise ValueError('smooth parameter has to be larger than 5')

        if smooth % 2 == 0:
            raise ValueError('smooth parameter has to be an odd number')

        dydx = savgol_filter(self.y, smooth, 3, deriv=1, delta=self.dx)
        angle = np.arctan(-dydx)
        return interp1d(self.x, angle, bounds_error=False,
                        fill_value='extrapolate', assume_sorted=True)

    def radius_of_curvature(self, smooth=5):
        """
        Returns the radius of curvature

        Parameters:

        smooth: int
            Window length in Savgol filter. Has to be odd.
        """
        if smooth < 5:
            raise ValueError('smooth parameter has to be larger than 5')

        if smooth % 2 == 0:
            raise ValueError('smooth parameter has to be an odd number')

        dy2dx2 = savgol_filter(self.y, smooth, 3, deriv=2, delta=self.dx)
        dydx = savgol_filter(self.y, smooth, 3, deriv=1, delta=self.dx)
        R = (1.0 + dydx**2)**1.5
        R /= dy2dx2
        return interp1d(self.x, R, bounds_error=False,
                        fill_value='extrapolate', assume_sorted=True)



from scipy.stats import linregress
import numpy as np


def extend_dataset(x, y, xmin, xmax, num_points=10):
    """
    Extends the dataset by fitting a linear curve to the end of
    the dataset.

    Parameters:

    x: array
        Original x array
    y: array
        Original y array
    xmin: float
        New minimum value for x
    xmax: float
        New maximum value for x
    num_points: int
        Number of points used in fit on each side

    Returns:

    x: array
        New extended x-array
    y: array
        New extended y-array
    """

    if xmin > np.min(x):
        raise ValueError("The new minimum value has to be smaller that the "
                         "current minimum value.")
    if xmax < np.max(x):
        raise ValueError("The new maximum value has to be larger than the "
                         "current maximum value.")

    if num_points > len(x):
        raise ValueError("num_points has to be smaller than the length of the "
                         "array")

    x1 = x[:num_points]
    y1 = y[:num_points]

    slope1, interscept1, pvalue, rvalue, stderr = linregress(x1, y1)

    new_y1 = slope1*xmin + interscept1
    x2 = x[-num_points:]
    y2 = y[-num_points:]
    slope2, interscept2, pvalue, rvalue, stderr = linregress(x1, y1)

    new_y2 = slope2*xmax + interscept2

    x_new = np.zeros(len(x)+2)
    y_new = np.zeros_like(x_new)
    x_new[1:-1] = x
    y_new[1:-1] = y
    x_new[0] = xmin
    x_new[-1] = xmax
    y_new[0] = new_y1
    y_new[-1] = new_y2
    return x_new, y_new




from setuptools import setup, find_packages

setup(
    name='eksternlab',
    description='Useful scripts for NTNU lab for engineers',
    version=1.3,
    author='David Kleiven',
    url='https://github.com/davidkleiven/eksternlab',
    download_url='https://github.com/davidkleiven/eksternlab/archive/v1.3.tar.gz',
    author_email='david.kleiven@ntnu.no',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 2.7'
    ],
    install_requires=['numpy', 'scipy']
)



import unittest
import numpy as np
from eksternlab import height, slope, curvature


SHOW = False


class TestSplines(unittest.TestCase):
    def test_interpolate(self):
        x = np.array([0.0, 0.2, 0.4, 0.6, 0.8, 1.0])
        y = x**2
        h = height(x, y)
        y_interp = h(x)
        self.assertTrue(np.allclose(y, y_interp))

        if SHOW:
            from matplotlib import pyplot as plt
            fig = plt.figure()
            ax = fig.add_subplot(1, 1, 1)
            ax.plot(x, y, 'x')
            x = np.linspace(-0.1, 1.1, 100)
            ax.plot(x, h(x))
            plt.show()

    def test_slope(self):
        x = np.array([-0.2, 0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2])
        y = x**2
        h = height(x, y)
        y_interp = h(x)

        x_interp = np.linspace(0.2, 0.8, 100)
        dydx = 2*x_interp
        angle = -np.arctan(dydx)
        s = slope(h, x_interp)

        if SHOW:
            from matplotlib import pyplot as plt
            fig = plt.figure()
            ax = fig.add_subplot(1, 1, 1)
            ax.plot(x_interp, angle, 'x')
            ax.plot(x_interp, s)
            plt.show()
        self.assertTrue(np.allclose(s, angle, atol=0.01))

    def test_curvature(self):
        x = np.array([-0.2, 0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2])
        y = x**2
        h = height(x, y)
        x_interp = np.linspace(0.2, 0.8, 100)
        dydx = 2*x_interp
        dy2dx2 = 2
        kappa = dy2dx2/(1 + dydx**2)**(1.5)
        kappa_interp = curvature(h, x_interp)

        if SHOW:
            from matplotlib import pyplot as plt
            fig = plt.figure()
            ax = fig.add_subplot(1, 1, 1)
            ax.plot(x_interp, kappa, 'x')
            ax.plot(x_interp, kappa_interp)
            plt.show()

        self.assertTrue(np.allclose(kappa, kappa_interp, atol=0.2))


if __name__ == '__main__':
    unittest.main()